@@ -75,14 +75,13 @@ START_TEST(service_thd2)
     TestEnv env;
     DummyGcs& conn(env.gcs());
     ServiceThd* thd = new ServiceThd(conn, env.gcache());
-    gu::UUID const state_uuid(NULL, 0);
     fail_if (thd == 0);
 
-    conn.set_last_applied(gu::GTID(state_uuid, 0));
+    conn.set_last_applied(0);
 
     gcs_seqno_t seqno = 1;
     thd->report_last_committed (seqno);
-    thd->flush(state_uuid);
+    thd->flush();
     WAIT_FOR(conn.last_applied() == seqno);
     fail_if (conn.last_applied() != seqno,
              "seqno = %" PRId64 ", expected %" PRId64, conn.last_applied(),
@@ -90,14 +89,14 @@ START_TEST(service_thd2)
 
     seqno = 5;
     thd->report_last_committed (seqno);
-    thd->flush(state_uuid);
+    thd->flush();
     WAIT_FOR(conn.last_applied() == seqno);
     fail_if (conn.last_applied() != seqno,
              "seqno = %" PRId64 ", expected %" PRId64, conn.last_applied(),
              seqno);
 
     thd->report_last_committed (3);
-    thd->flush(state_uuid);
+    thd->flush();
     WAIT_FOR(conn.last_applied() == seqno);
     fail_if (conn.last_applied() != seqno,
              "seqno = %" PRId64 ", expected %" PRId64, conn.last_applied(),
@@ -107,7 +106,7 @@ START_TEST(service_thd2)
 
     seqno = 3;
     thd->report_last_committed (seqno);
-    thd->flush(state_uuid);
+    thd->flush();
     WAIT_FOR(conn.last_applied() == seqno);
     fail_if (conn.last_applied() != seqno,
              "seqno = %" PRId64 ", expected %" PRId64, conn.last_applied(),
@@ -139,6 +138,7 @@ Suite* service_thd_suite()
     tcase_add_test  (tc, service_thd1);
     tcase_add_test  (tc, service_thd2);
     tcase_add_test  (tc, service_thd3);
+    tcase_set_timeout(tc, 60);
     suite_add_tcase (s, tc);
 
     return s;
