@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2014 Codership Oy <info@codership.com>
+ * Copyright (C) 2010-2018 Codership Oy <info@codership.com>
  */
 
 #include "write_set.hpp"
@@ -24,7 +24,7 @@ namespace
             GCache_setup(gu::Config& conf) : name_("write_set_test.gcache")
             {
                 conf.set("gcache.name", name_);
-                conf.set("gcache.size", "4M");
+                conf.set("gcache.size", "1M");
                 log_info << "conf for gcache: " << conf;
             }
 
@@ -437,8 +437,8 @@ START_TEST(test_cert_hierarchical_v1)
 
         trx->set_received(0, wsi[i].local_seqno, wsi[i].global_seqno);
         Certification::TestResult result(cert.append_trx(trx));
-        fail_unless(result == wsi[i].result, "g: %lld r: %d er: %d",
-                    trx->global_seqno(), result, wsi[i].result);
+        fail_unless(result == wsi[i].result, "wsi: %zu, g: %lld r: %d er: %d",
+                    i, trx->global_seqno(), result, wsi[i].result);
         fail_unless(trx->depends_seqno() == wsi[i].expected_depends_seqno,
                     "wsi: %zu g: %lld ld: %lld eld: %lld",
                     i, trx->global_seqno(), trx->depends_seqno(),
@@ -573,7 +573,11 @@ START_TEST(test_cert_hierarchical_v2)
 }
 END_TEST
 
-
+// This test leaks memory and it is for trx protocol version 2
+// which is pre 25.3.5. Disabling this test for now with ASAN
+// build. The test should be either removed or fixed to work
+// with more recent protocol versions.
+#ifndef GALERA_WITH_ASAN
 START_TEST(test_trac_726)
 {
     log_info << "test_trac_726";
@@ -641,7 +645,7 @@ START_TEST(test_trac_726)
     }
 }
 END_TEST
-
+#endif // GALERA_WITH_ASAN
 
 Suite* write_set_suite()
 {
@@ -675,13 +679,15 @@ Suite* write_set_suite()
 
     tc = tcase_create("test_cert_hierarchical_v2");
     tcase_add_test(tc, test_cert_hierarchical_v2);
-    tcase_set_timeout(tc, 20);
+    tcase_set_timeout(tc, 120);
     suite_add_tcase(s, tc);
 
+#ifndef GALERA_WITH_ASAN
     tc = tcase_create("test_trac_726");
     tcase_add_test(tc, test_trac_726);
     tcase_set_timeout(tc, 20);
     suite_add_tcase(s, tc);
+#endif
 
     return s;
 }
