@@ -19,9 +19,6 @@ public:
     bool
     operator ==(const Test& other) const { return val_ == other.val_; }
 
-    bool
-    operator !=(const Test& other) const { return !operator==(other); }
-
     int
     operator +() const { return val_; }
 
@@ -37,19 +34,19 @@ START_TEST(ctor_clear)
     typedef gu::DeqMap<signed char, Test> Map;
     Map m(-1);
 
-    ck_assert(m.size() <= 0);
-    ck_assert(m.index_begin() == Map::index_type(-1));
-    ck_assert(m.index_end() == Map::index_type(-1));
+    fail_if(m.size() > 0);
+    fail_if(m.index_begin() != Map::index_type(-1));
+    fail_if(m.index_end() != Map::index_type(-1));
 
-    ck_assert(m.upper_bound(0) == m.index_end());
+    fail_if(m.upper_bound(0) != m.index_end());
 
     m.clear(5);
 
-    ck_assert(m.size() <= 0);
-    ck_assert(m.index_begin() == Map::index_type(5));
-    ck_assert(m.index_end() == Map::index_type(5));
+    fail_if(m.size() > 0);
+    fail_if(m.index_begin() != Map::index_type(5));
+    fail_if(m.index_end() != Map::index_type(5));
 
-    ck_assert(m.upper_bound(0) == m.index_begin());
+    fail_if(m.upper_bound(0) != m.index_begin());
 }
 END_TEST
 
@@ -65,51 +62,51 @@ START_TEST(push_pop)
     m.push_front(4);// -2
     /* Here we have: 4, 1, 2, 3 */
 
-    ck_assert(m.size()  == 4);
-    ck_assert(m.front() == 4);
-    ck_assert(m.back()  == 3);
-    ck_assert(m.index_begin() == Map::index_type(-2));
-    ck_assert(m.index_end()   == Map::index_type(2));
+    fail_if(m.size()  != 4);
+    fail_if(m.front() != 4);
+    fail_if(m.back()  != 3);
+    fail_if(m.index_begin() != Map::index_type(-2));
+    fail_if(m.index_end()   != Map::index_type(2));
 
     m.pop_front();
     /* Here we have: 1, 2, 3 */
 
-    ck_assert(m.size()  == 3);
-    ck_assert(m.front() == 1);
-    ck_assert(m.back()  == 3);
-    ck_assert(m.index_begin() == Map::index_type(-1));
-    ck_assert(m.index_end()   == Map::index_type(2));
+    fail_if(m.size()  != 3);
+    fail_if(m.front() != 1);
+    fail_if(m.back()  != 3);
+    fail_if(m.index_begin() != Map::index_type(-1));
+    fail_if(m.index_end()   != Map::index_type(2));
 
     m.pop_back();
     /* Here we have: 1, 2 */
 
-    ck_assert(m.size()  == 2);
-    ck_assert(m.front() == 1);
-    ck_assert(m.back()  == 2);
-    ck_assert(m.index_begin() == Map::index_type(-1));
-    ck_assert(m.index_end()   == Map::index_type(1));
+    fail_if(m.size()  != 2);
+    fail_if(m.front() != 1);
+    fail_if(m.back()  != 2);
+    fail_if(m.index_begin() != Map::index_type(-1));
+    fail_if(m.index_end()   != Map::index_type(1));
 
     m.pop_front();
     m.pop_front();
     /* Here we have: empty */
 
-    ck_assert(m.size()  == 0);
-    ck_assert(m.index_begin() == Map::index_type(1));
-    ck_assert(m.index_end()   == Map::index_type(1));
+    fail_if(m.size()  != 0);
+    fail_if(m.index_begin() != Map::index_type(1));
+    fail_if(m.index_end()   != Map::index_type(1));
 
     m.push_front(7); // 0
 
-    ck_assert(m.size()  == 1);
-    ck_assert(m.front() == 7);
-    ck_assert(m.back()  == 7);
-    ck_assert(m.index_begin() == Map::index_type(0));
-    ck_assert(m.index_end()   == Map::index_type(1));
+    fail_if(m.size()  != 1);
+    fail_if(m.front() != 7);
+    fail_if(m.back()  != 7);
+    fail_if(m.index_begin() != Map::index_type(0));
+    fail_if(m.index_end()   != Map::index_type(1));
 
     m.pop_back();
 
-    ck_assert(m.size()  == 0);
-    ck_assert(m.index_begin() == Map::index_type(0));
-    ck_assert(m.index_end()   == Map::index_type(0));
+    fail_if(m.size()  != 0);
+    fail_if(m.index_begin() != Map::index_type(0));
+    fail_if(m.index_end()   != Map::index_type(0));
 }
 END_TEST
 
@@ -118,45 +115,45 @@ START_TEST(pop_holes) /* autoshrinking when popping on container with holes */
     typedef gu::DeqMap<signed char, signed char> Map;
     Map m(-1);
 
-    ck_assert(m.size() == 0);
-    ck_assert(m.index_begin() == Map::index_type(-1));
-    ck_assert(m.index_end()   == Map::index_type(-1));
-    ck_assert(m.index_back()  == Map::index_type(-2));
+    fail_if(m.size() != 0);
+    fail_if(m.index_begin() != Map::index_type(-1));
+    fail_if(m.index_end()   != Map::index_type(-1));
+    fail_if(m.index_back()  != Map::index_type(-2));
 
     m.insert(1, 1);
     m.insert(4, 4);
 
-    ck_assert(m.size() == 4);
-    ck_assert(m.index_begin() == Map::index_type(1));
-    ck_assert(m.index_end()   == Map::index_type(5));
-    ck_assert(m.index_back()  == Map::index_type(4));
-    ck_assert(Map::not_set(m[2]));
-    ck_assert(Map::not_set(m[3]));
+    fail_if(m.size() != 4, "expected 4 got %u", m.size());
+    fail_if(m.index_begin() != Map::index_type(1));
+    fail_if(m.index_end()   != Map::index_type(5));
+    fail_if(m.index_back()  != Map::index_type(4));
+    fail_if(!Map::not_set(m[2]));
+    fail_if(!Map::not_set(m[3]));
 
     m.pop_front();
 
-    ck_assert(m.size() == 1);
-    ck_assert(m.index_begin() == Map::index_type(4));
-    ck_assert(m.index_end()   == Map::index_type(5));
-    ck_assert(m.index_back()  == Map::index_type(4));
-    ck_assert(*m.begin() == 4);
+    fail_if(m.size() != 1);
+    fail_if(m.index_begin() != Map::index_type(4));
+    fail_if(m.index_end()   != Map::index_type(5));
+    fail_if(m.index_back()  != Map::index_type(4));
+    fail_if(*m.begin() != 4);
 
     m.insert(1, 1);
 
-    ck_assert(m.size() == 4);
-    ck_assert(m.index_begin() == Map::index_type(1));
-    ck_assert(m.index_end()   == Map::index_type(5));
-    ck_assert(m.index_back()  == Map::index_type(4));
-    ck_assert(Map::not_set(m[2]));
-    ck_assert(Map::not_set(m[3]));
+    fail_if(m.size() != 4);
+    fail_if(m.index_begin() != Map::index_type(1));
+    fail_if(m.index_end()   != Map::index_type(5));
+    fail_if(m.index_back()  != Map::index_type(4));
+    fail_if(!Map::not_set(m[2]));
+    fail_if(!Map::not_set(m[3]));
 
     m.pop_back();
 
-    ck_assert(m.size() == 1);
-    ck_assert(m.index_begin() == Map::index_type(1));
-    ck_assert(m.index_end()   == Map::index_type(2));
-    ck_assert(m.index_back()  == Map::index_type(1));
-    ck_assert(*m.begin() == 1);
+    fail_if(m.size() != 1);
+    fail_if(m.index_begin() != Map::index_type(1));
+    fail_if(m.index_end()   != Map::index_type(2));
+    fail_if(m.index_back()  != Map::index_type(1));
+    fail_if(*m.begin() != 1);
 }
 END_TEST
 
@@ -168,7 +165,7 @@ START_TEST(at)
     try
     {
         m.at(-1);
-        ck_abort_msg("expected exception");
+        fail("expected exception");
     }
     catch (gu::NotFound&) {}
 
@@ -176,24 +173,24 @@ START_TEST(at)
 
     try
     {
-        ck_assert(3 == m.at(-1));
+        fail_if(3 != m.at(-1));
     }
     catch (...)
     {
-        ck_abort_msg("unexpected exception");
+        fail("unexpected exception");
     }
 
     try
     {
         m.at(-2);
-        ck_abort_msg("expected exception");
+        fail("expected exception");
     }
     catch (gu::NotFound&) {}
 
     try
     {
         m.at(0);
-        ck_abort_msg("expected exception");
+        fail("expected exception");
     }
     catch (gu::NotFound&) {}
 }
@@ -207,98 +204,98 @@ START_TEST(iterators_insert)
     m.insert(m.begin(), 4, 4);
     /* here we have 4, 4, 4, 4 */
 
-    ck_assert(m.size() == 4);
-    ck_assert(*m.begin()  == 4);
-    ck_assert(*m.rbegin() == 4);
-    ck_assert(m.index_begin() == Map::index_type(-1));
-    ck_assert(m.index_end()   == Map::index_type(3));
+    fail_if(m.size() != 4);
+    fail_if(*m.begin()  != 4);
+    fail_if(*m.rbegin() != 4);
+    fail_if(m.index_begin() != Map::index_type(-1));
+    fail_if(m.index_end()   != Map::index_type(3));
 
     m.insert(m.begin() + 1, 2, 2); // bulk insert (overwrite) in the middle
     /* here we have 4, 2, 2, 4 */
 
-    ck_assert(m.size() == 4);
-    ck_assert(*m.begin()  == 4);
-    ck_assert(*m.rbegin() == 4);
-    ck_assert(m.index_begin() == Map::index_type(-1));
-    ck_assert(m.index_end()   == Map::index_type(3));
+    fail_if(m.size() != 4);
+    fail_if(*m.begin()  != 4);
+    fail_if(*m.rbegin() != 4);
+    fail_if(m.index_begin() != Map::index_type(-1));
+    fail_if(m.index_end()   != Map::index_type(3));
 
     m.insert(m.begin(), 2, 1); // bulk insert (overwrite) in the beginning
     /* here we have 1, 1, 2, 4 */
 
-    ck_assert(m.size() == 4);
-    ck_assert(*m.begin()  == 1);
-    ck_assert(*m.rbegin() == 4);
-    ck_assert(m.index_begin() == Map::index_type(-1));
-    ck_assert(m.index_end()   == Map::index_type(3));
+    fail_if(m.size() != 4);
+    fail_if(*m.begin()  != 1);
+    fail_if(*m.rbegin() != 4);
+    fail_if(m.index_begin() != Map::index_type(-1));
+    fail_if(m.index_end()   != Map::index_type(3));
 
     m.insert(m.end(), 2, 5); // bulk insert in the end
     /* here we have 1, 1, 2, 4, 5, 5 */
 
-    ck_assert(m.size() == 6);
-    ck_assert(*m.begin()  == 1);
-    ck_assert(*m.rbegin() == 5);
-    ck_assert(m.index_begin() == Map::index_type(-1));
-    ck_assert(m.index_end()   == Map::index_type(5));
+    fail_if(m.size() != 6);
+    fail_if(*m.begin()  != 1);
+    fail_if(*m.rbegin() != 5);
+    fail_if(m.index_begin() != Map::index_type(-1));
+    fail_if(m.index_end()   != Map::index_type(5));
 
     m.insert(m.begin(), -1); // single insert (overwrite) in the beginning
     /* here we have -1, 1, 2, 4, 5, 5 */
 
-    ck_assert(m.size() == 6);
-    ck_assert(*m.begin()  == -1);
-    ck_assert(*m.rbegin() == 5);
-    ck_assert(m.index_begin() == Map::index_type(-1));
-    ck_assert(m.index_end()   == Map::index_type(5));
+    fail_if(m.size() != 6);
+    fail_if(*m.begin()  != -1);
+    fail_if(*m.rbegin() != 5);
+    fail_if(m.index_begin() != Map::index_type(-1));
+    fail_if(m.index_end()   != Map::index_type(5));
 
     Map::iterator b(m.begin());
     ++b;
-    ck_assert(!Map::not_set(*b));
-    ck_assert(*b == 1);
-    ck_assert(m.index(b) == 0);
+    fail_if(Map::not_set(*b));
+    fail_if(*b != 1);
+    fail_if(m.index(b) != 0);
     ++b;
-    ck_assert(*b == 2);
-    ck_assert(m.index(b) == 1);
+    fail_if(*b != 2);
+    fail_if(m.index(b) != 1);
 
     m.insert(b, 1); // single insert (overwrite) in the middle
     /* here we have -1, 1, 1, 4, 5, 5 */
 
-    ck_assert(m.size() == 6);
-    ck_assert(*b == 1);
+    fail_if(m.size() != 6);
+    fail_if(*b != 1);
 
     m.insert(m.end(), 6); // single insert in the end
     /* here we have -1, 1, 1, 4, 5, 5, 6 */
 
-    ck_assert(m.size() == 7);
-    ck_assert(*m.begin()  == -1);
-    ck_assert(*m.rbegin() == 6);
-    ck_assert(m.index_begin() == Map::index_type(-1));
-    ck_assert(m.index_end()   == Map::index_type(6));
-    ck_assert(m.index_back()  == Map::index_type(5));
-    ck_assert(m[m.index_back()] == 6);
+    fail_if(m.size() != 7);
+    fail_if(*m.begin()  != -1);
+    fail_if(*m.rbegin() != 6);
+    fail_if(m.index_begin() != Map::index_type(-1));
+    fail_if(m.index_end()   != Map::index_type(6));
+    fail_if(m.index_back()  != Map::index_type(5));
+    fail_if(m[m.index_back()] != 6);
 
     ++b;
-    ck_assert(*b == 4);
-    ck_assert(m.index(b) == 2);
+    fail_if(*b != 4);
+    fail_if(m.index(b) != 2);
     *b = 2; // assignment via iterator
     /* here we have -1, 1, 1, 2, 5, 5, 6 */
 
-    ck_assert(m.size() == 7);
-    ck_assert(*b == 2);
+    fail_if(m.size() != 7);
+    fail_if(*b != 2);
 
     Map::reverse_iterator rb(m.rbegin());
-    ck_assert(*rb == 6);
-    ck_assert(m.index(rb) == 5);
+    fail_if(*rb != 6);
+    fail_if(m.index(rb) != 5);
 
     *rb = 5;
     /* here we have -1, 1, 1, 2, 5, 5, 5 */
-    ck_assert(*rb == 5);
+    fail_if(*rb != 5);
 
     ++rb;
-    ck_assert(*rb == 5);
-    ck_assert(m.index(rb) == 4);
+    fail_if(*rb != 5);
+    fail_if(m.index(rb) != 4);
 
     *rb = 4;
     /* here we have -1, 1, 1, 2, 5, 4, 5 */
-    ck_assert(*rb == 4);
+    fail_if(*rb != 4);
 }
 END_TEST
 
@@ -311,143 +308,143 @@ START_TEST(iterators_erase)
     m.insert(m.begin(), init_size, 1);
     /* here we have 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 */
 
-    ck_assert(m.size() == init_size);
-    ck_assert(m.index_begin() == Map::index_type(-1));
-    ck_assert(m.index_end() == m.index_begin() + int(m.size()));
-    ck_assert(m.front() == 1);
-    ck_assert(m.back()  == 1);
+    fail_if(m.size() != init_size);
+    fail_if(m.index_begin() != Map::index_type(-1));
+    fail_if(m.index_end() != m.index_begin() + int(m.size()));
+    fail_if(m.front() != 1);
+    fail_if(m.back()  != 1);
 
     Map::iterator b(m.begin() + 2);
     m.erase(b); // single erase in the middle
     /* here we have 1, 1, N, 1, 1, 1, 1, 1, 1, 1, 1, 1 */
     /*                    b                            */
-    ck_assert(Map::not_set(*b));
+    fail_unless(Map::not_set(*b));
 
     Map::iterator e(m.end() - 4);
     m.erase(e, e + 2); // bulk erase in the middle
     /* here we have 1, 1, N, 1, 1, 1, 1, 1, N, N, 1, 1 */
     /*                    b                 e          */
-    ck_assert(Map::not_set(*e));
+    fail_unless(Map::not_set(*e));
 
-    ck_assert(m.size() == init_size);
-    ck_assert(m.index_begin() == Map::index_type(-1));
-    ck_assert(m.index_end() == m.index_begin() + int(m.size()));
-    ck_assert(m.front() == 1);
-    ck_assert(m.back()  == 1);
+    fail_if(m.size() != init_size);
+    fail_if(m.index_begin() != Map::index_type(-1));
+    fail_if(m.index_end() != m.index_begin() + int(m.size()));
+    fail_if(m.front() != 1);
+    fail_if(m.back()  != 1);
 
     m.erase(m.begin()); // single erase at the front
     /* here we have    1, N, 1, 1, 1, 1, 1, N, N, 1, 1 */
     /*                    b                 e          */
 
-    ck_assert(m.size() == init_size - 1);
-    ck_assert(m.index_begin() == Map::index_type(0));
-    ck_assert(m.index_end() == m.index_begin() + int(m.size()));
-    ck_assert(m.front() == 1);
-    ck_assert(m.back()  == 1);
+    fail_if(m.size() != init_size - 1);
+    fail_if(m.index_begin() != Map::index_type(0));
+    fail_if(m.index_end() != m.index_begin() + int(m.size()));
+    fail_if(m.front() != 1);
+    fail_if(m.back()  != 1);
 
     m.erase(m.begin()); // single erase at the front before hole
     /* here we have          1, 1, 1, 1, 1, N, N, 1, 1 */
     /*                    b                 e          */
 
-    ck_assert(m.size() == init_size - 3);
-    ck_assert(m.index_begin() == Map::index_type(2));
-    ck_assert(m.index_end() == m.index_begin() + int(m.size()));
-    ck_assert(m.front() == 1);
-    ck_assert(m.back()  == 1);
+    fail_if(m.size() != init_size - 3);
+    fail_if(m.index_begin() != Map::index_type(2));
+    fail_if(m.index_end() != m.index_begin() + int(m.size()));
+    fail_if(m.front() != 1);
+    fail_if(m.back()  != 1);
 
     m.erase(m.end() - 1); // single erase at the back
     /* here we have          1, 1, 1, 1, 1, N, N, 1    */
     /*                    b                 e          */
 
-    ck_assert(m.size() == init_size - 4);
-    ck_assert(m.index_begin() == Map::index_type(2));
-    ck_assert(m.index_end() == m.index_begin() + int(m.size()));
-    ck_assert(m.front() == 1);
-    ck_assert(m.back()  == 1);
+    fail_if(m.size() != init_size - 4);
+    fail_if(m.index_begin() != Map::index_type(2));
+    fail_if(m.index_end() != m.index_begin() + int(m.size()));
+    fail_if(m.front() != 1);
+    fail_if(m.back()  != 1);
 
     m.erase(m.end() - 1); // single erase at the back before hole
     /* here we have          1, 1, 1, 1, 1             */
     /*                    b                 e          */
 
-    ck_assert(m.size() == init_size - 7);
-    ck_assert(m.index_begin() == Map::index_type(2));
-    ck_assert(m.index_end() == m.index_begin() + int(m.size()));
-    ck_assert(m.front() == 1);
-    ck_assert(m.back()  == 1);
+    fail_if(m.size() != init_size - 7);
+    fail_if(m.index_begin() != Map::index_type(2));
+    fail_if(m.index_end() != m.index_begin() + int(m.size()));
+    fail_if(m.front() != 1);
+    fail_if(m.back()  != 1);
 
     m.erase(m.begin(), m.begin() + 2); // bulk erase at the front
     /* here we have                1, 1, 1             */
     /*                    b                 e          */
 
-    ck_assert(m.size() == init_size - 9);
-    ck_assert(m.index_begin() == Map::index_type(4));
-    ck_assert(m.index_end() == m.index_begin() + int(m.size()));
-    ck_assert(m.front() == 1);
-    ck_assert(m.back()  == 1);
+    fail_if(m.size() != init_size - 9);
+    fail_if(m.index_begin() != Map::index_type(4));
+    fail_if(m.index_end() != m.index_begin() + int(m.size()));
+    fail_if(m.front() != 1);
+    fail_if(m.back()  != 1);
 
     m.erase(m.end() - 2, m.end()); // bulk erase at the end
     /* here we have                1                   */
     /*                    b                 e          */
 
-    ck_assert(m.size() == init_size - 11);
-    ck_assert(m.index_begin() == Map::index_type(4));
-    ck_assert(m.index_end() == m.index_begin() + int(m.size()));
-    ck_assert(m.front() == 1);
-    ck_assert(m.back()  == 1);
+    fail_if(m.size() != init_size - 11);
+    fail_if(m.index_begin() != Map::index_type(4));
+    fail_if(m.index_end() != m.index_begin() + int(m.size()));
+    fail_if(m.front() != 1);
+    fail_if(m.back()  != 1);
 
     m.insert(m.end(), 16, 1);
     init_size = m.size();
-    ck_assert (init_size == 17);
+    fail_if (init_size != 17);
     /* here we have 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 */
 
     m.erase(m.begin() + 1, m.begin() + 3);
     /* here we have 1, N, N, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 */
-    ck_assert(m.size() == init_size);
+    fail_if(m.size() != init_size);
 
     m.erase(m.begin() + 2, m.begin() + 4);
     /* here we have 1, N, N, N, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 */
-    ck_assert(m.size() == init_size);
+    fail_if(m.size() != init_size);
 
     m.erase(m.begin(), m.begin() + 2);
     /* here we have             1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 */
-    ck_assert(m.size() == init_size - 4);
-    ck_assert(m.index_begin() == Map::index_type(8));
+    fail_if(m.size() != init_size - 4);
+    fail_if(m.index_begin() != Map::index_type(8));
 
     m.erase(m.end() - 3, m.end() - 1);
     /* here we have             1, 1, 1, 1, 1, 1, 1, 1, 1, 1, N, N, 1 */
-    ck_assert(m.size() == init_size - 4);
+    fail_if(m.size() != init_size - 4);
 
     m.erase(m.end() - 4, m.end() - 1);
     /* here we have             1, 1, 1, 1, 1, 1, 1, 1, 1, N, N, N, 1 */
-    ck_assert(m.size() == init_size - 4);
+    fail_if(m.size() != init_size - 4);
 
     m.erase(m.end() - 2, m.end());
     /* here we have             1, 1, 1, 1, 1, 1, 1, 1, 1             */
-    ck_assert(m.size() == init_size - 8);
+    fail_if(m.size() != init_size - 8);
 
     m.erase(m.begin() + 2, m.begin() + 4);
     /* here we have             1, 1, N, N, 1, 1, 1, 1, 1             */
-    ck_assert(m.size() == init_size - 8);
+    fail_if(m.size() != init_size - 8);
 
     m.erase(m.begin() + 5, m.begin() + 7);
     /* here we have             1, 1, N, N, 1, N, N, 1, 1             */
-    ck_assert(m.size() == init_size - 8);
-    ck_assert(m.index_begin() == Map::index_type(8));
+    fail_if(m.size() != init_size - 8);
+    fail_if(m.index_begin() != Map::index_type(8));
 
-    ck_assert(Map::not_set(m[m.index_begin() + 2]));
-    ck_assert(Map::not_set(m[m.index_begin() + 3]));
-    ck_assert(!Map::not_set(m[m.index_begin() + 4]));
-    ck_assert(Map::not_set(m[m.index_begin() + 5]));
-    ck_assert(Map::not_set(m[m.index_begin() + 6]));
+    fail_unless(Map::not_set(m[m.index_begin() + 2]));
+    fail_unless(Map::not_set(m[m.index_begin() + 3]));
+    fail_if    (Map::not_set(m[m.index_begin() + 4]));
+    fail_unless(Map::not_set(m[m.index_begin() + 5]));
+    fail_unless(Map::not_set(m[m.index_begin() + 6]));
 
     m.erase(m.begin() + 1, m.begin() + 8);
     /* here we have             1, N, N, N, N, N, N, N, 1             */
-    ck_assert(m.size() == init_size - 8);
+    fail_if(m.size() != init_size - 8);
 
     m.erase(m.begin());
     /* here we have                                     1             */
-    ck_assert(m.size() == 1);
-    ck_assert(m.index_begin() == Map::index_type(16));
+    fail_if(m.size() != 1);
+    fail_if(m.index_begin() != Map::index_type(16));
 }
 END_TEST
 
@@ -458,143 +455,143 @@ null_insertions(size_t const SIZE)
 {
     typedef gu::DeqMap<signed char, Test> Map;
     Map::value_type const Null(Map::null_value());
-    ck_assert(Map::null_value() == Null);
-    ck_assert(Map::not_set(Null));
+    fail_unless(Map::null_value() == Null);
+    fail_unless(Map::not_set(Null));
 
     Map::value_type const Default = Map::value_type(0);
-    ck_assert(Null != Default);
-    ck_assert(!Map::not_set(Default));
+    fail_if(Null == Default);
+    fail_if(Map::not_set(Default));
 
     Map::index_type const Begin(-1);
     Map m(Begin);
 
     m.insert(m.end(), SIZE, Default);
 
-    ck_assert(m.size() == SIZE);
-    ck_assert(m.index_begin() == Begin);
-    ck_assert(m.index_end()   == m.index_begin() + int(m.size()));
-    ck_assert((m.size() == 0 || m.front() == Default));
-    ck_assert((m.size() == 0 || m.back()  == Default));
+    fail_if(m.size() != SIZE);
+    fail_if(m.index_begin() != Begin);
+    fail_if(m.index_end()   != m.index_begin() + int(m.size()));
+    fail_unless((m.size() == 0 || m.front() == Default));
+    fail_unless((m.size() == 0 || m.back()  == Default));
 
     try
     {
         m.push_front(Null);
-        ck_abort_msg("No exception in push_front()");
+        fail("No exception in push_front()");
     }
     catch (std::invalid_argument& e)
     {
         try
         {
             m.push_back(Null);
-            ck_abort_msg("No exception in push_back()");
+            fail("No exception in push_back()");
         }
         catch (std::invalid_argument& e)
         {
-            ck_assert(m.size() == SIZE);
-            ck_assert(m.index_begin() == Begin);
-            ck_assert(m.index_end()   == m.index_begin() + int(m.size()));
-            ck_assert(m.size() == 0 || m.front() == Default);
-            ck_assert(m.size() == 0 || m.back()  == Default);
+            fail_if(m.size() != SIZE);
+            fail_if(m.index_begin() != Begin);
+            fail_if(m.index_end()   != m.index_begin() + int(m.size()));
+            fail_unless(m.size() == 0 || m.front() == Default);
+            fail_unless(m.size() == 0 || m.back()  == Default);
         }
         catch (...)
         {
-            ck_abort_msg("Unexpected exception in push_back()");
+            fail("Unexpected exception in push_back()");
         }
     }
     catch (...)
     {
-        ck_abort_msg("Unexpected exception in push_front()");
+        fail("Unexpected exception in push_front()");
     }
 
     try
     {
         m.insert(m.begin(), Null);
-        ck_abort_msg("No exception in insert() at the begin()");
+        fail("No exception in insert() at the begin()");
     }
     catch (std::invalid_argument& e)
     {
         try
         {
             m.insert(m.end(), Null);
-            ck_abort_msg("No exception in insert() at the end()");
+            fail("No exception in insert() at the end()");
         }
         catch (std::invalid_argument& e)
         {
-            ck_assert(m.size() == SIZE);
-            ck_assert(m.index_begin() == Begin);
-            ck_assert(m.index_end()   == m.index_begin() + int(m.size()));
-            ck_assert(m.size() == 0 || m.front() == Default);
-            ck_assert(m.size() == 0 || m.back()  == Default);
+            fail_if(m.size() != SIZE);
+            fail_if(m.index_begin() != Begin);
+            fail_if(m.index_end()   != m.index_begin() + int(m.size()));
+            fail_unless(m.size() == 0 || m.front() == Default);
+            fail_unless(m.size() == 0 || m.back()  == Default);
         }
         catch (...)
         {
-            ck_abort_msg("Unexpected exception in insert() at the end()");
+            fail("Unexpected exception in insert() at the end()");
         }
     }
     catch (...)
     {
-        ck_abort_msg("Unexpected exception in insert() at the begin()");
+        fail("Unexpected exception in insert() at the begin()");
     }
 
     try
     {
         m.insert(m.begin(), 3, Null);
-        ck_abort_msg("No exception in insert() at the begin()");
+        fail("No exception in insert() at the begin()");
     }
     catch (std::invalid_argument& e)
     {
         try
         {
             m.insert(m.end(), 3, Null);
-            ck_abort_msg("No exception in insert() at the end()");
+            fail("No exception in insert() at the end()");
         }
         catch (std::invalid_argument& e)
         {
-            ck_assert(m.size() == SIZE);
-            ck_assert(m.index_begin() == Begin);
-            ck_assert(m.index_end()   == m.index_begin() + int(m.size()));
-            ck_assert(m.size() == 0 || m.front() == Default);
-            ck_assert(m.size() == 0 || m.back()  == Default);
+            fail_if(m.size() != SIZE);
+            fail_if(m.index_begin() != Begin);
+            fail_if(m.index_end()   != m.index_begin() + int(m.size()));
+            fail_unless(m.size() == 0 || m.front() == Default);
+            fail_unless(m.size() == 0 || m.back()  == Default);
         }
         catch (...)
         {
-            ck_abort_msg("Unexpected exception in insert() at the end()");
+            fail("Unexpected exception in insert() at the end()");
         }
     }
     catch (...)
     {
-        ck_abort_msg("Unexpected exception in insert() at the begin()");
+        fail("Unexpected exception in insert() at the begin()");
     }
 
 
     try
     {
         m.insert(m.index_begin(), Null);
-        ck_abort_msg("No exception in insert() at the begin()");
+        fail("No exception in insert() at the begin()");
     }
     catch (std::invalid_argument& e)
     {
         try
         {
             m.insert(m.index_end(), Null);
-            ck_abort_msg("No exception in insert() at the end()");
+            fail("No exception in insert() at the end()");
         }
         catch (std::invalid_argument& e)
         {
-            ck_assert(m.size() == SIZE);
-            ck_assert(m.index_begin() == Begin);
-            ck_assert(m.index_end()   == m.index_begin() + int(m.size()));
-            ck_assert(m.size() == 0 || m.front() == Default);
-            ck_assert(m.size() == 0 || m.back()  == Default);
+            fail_if(m.size() != SIZE);
+            fail_if(m.index_begin() != Begin);
+            fail_if(m.index_end()   != m.index_begin() + int(m.size()));
+            fail_unless(m.size() == 0 || m.front() == Default);
+            fail_unless(m.size() == 0 || m.back()  == Default);
         }
         catch (...)
         {
-            ck_abort_msg("Unexpected exception in insert() at the end()");
+            fail("Unexpected exception in insert() at the end()");
         }
     }
     catch (...)
     {
-        ck_abort_msg("Unexpected exception in insert() at the begin()");
+        fail("Unexpected exception in insert() at the begin()");
     }
 }
 
@@ -620,15 +617,15 @@ START_TEST(random_access)
     m.insert(Min, Test(Min));
     m.insert(Max, Test(Max));
 
-    ck_assert(m.size() == size_t(Max - Min + 1));
-    ck_assert(m.index_begin() == Min);
-    ck_assert(m.index_back()  == Max);
-    ck_assert(m.front() == Test(Min));
-    ck_assert(m.back()  == Test(Max));
+    fail_if(m.size() != size_t(Max - Min + 1));
+    fail_if(m.index_begin() != Min);
+    fail_if(m.index_back()  != Max);
+    fail_unless(m.front() == Test(Min));
+    fail_unless(m.back()  == Test(Max));
 
     for (Map::index_type i(Min + 1); i < Max; ++i)
     {
-        ck_assert(Map::not_set(m[i]));
+        fail_unless(Map::not_set(m[i]));
     }
 
     for (Map::index_type i(Min + 1); i < Max; ++i)
@@ -642,8 +639,8 @@ START_TEST(random_access)
     {
         Map::value_type const val((Test(i)));
 
-        ck_assert(m[i] == val);
-        if (!Map::not_set(val)) ck_assert(m.at(i) == val);
+        fail_unless(m[i] == val);
+        if (!Map::not_set(val)) fail_unless(m.at(i) == val);
     }
 }
 END_TEST
@@ -657,20 +654,20 @@ START_TEST(find)
     m.insert(2, 2);
     m.insert(3, 3);
 
-    ck_assert(m.size() == 3);
-    ck_assert(m[1] == 1);
-    ck_assert(m[3] == 3);
+    fail_if(m.size()  != 3);
+    fail_if(m[1] != 1);
+    fail_if(m[3] != 3);
 
     for(Map::index_type i(m.index_begin()); i < m.index_end(); ++i)
     {
-        ck_assert(*m.find(i) == i);
+        fail_if(*m.find(i) != i);
         *m.find(i) = i + 1;
-        ck_assert(*m.find(i) == i + 1);
+        fail_if(*m.find(i) != i + 1);
     }
     const Map& mc(m); // test const overload
 
     for(Map::index_type i(mc.index_begin()); i < mc.index_end(); ++i)
-        ck_assert(*mc.find(i) == i + 1);
+        fail_if(*mc.find(i) != i + 1);
 }
 END_TEST
 
@@ -764,19 +761,19 @@ START_TEST(random_test)
         {
             Map::iterator it(map.begin() + (idx - begin));
             it = map.insert(it, val);
-            ck_assert(it != map.end());
+            fail_if(it == map.end());
             if (size_change)
-                ck_assert(init_state != map_state(map));
+                fail_if(init_state == map_state(map));
             else
-                ck_assert(init_state == map_state(map));
+                fail_if(init_state != map_state(map));
         }
         break;
         case INDEX:
             map.insert(idx, val);
             if (size_change)
-                ck_assert(init_state != map_state(map));
+                fail_if(init_state == map_state(map));
             else
-                ck_assert(init_state == map_state(map));
+                fail_if(init_state != map_state(map));
 
             break;
         case PUSHPOP:
@@ -784,27 +781,27 @@ START_TEST(random_test)
                 map.push_front(val);
             else
             {
-                ck_assert(idx != end);
+                fail_if(idx != end);
                 map.push_back(val);
             }
 
-            ck_assert(init_state != map_state(map));
+            fail_if(init_state == map_state(map));
         }
     }
 
-    ck_assert(!map.empty());
+    fail_if(map.empty());
 
     /* now erase all elements */
     while (!map.empty())
     {
         int const begin(map.index_begin());
         int const end(map.index_end());
-        ck_assert(begin < end);
-        ck_assert(map.index_back() < end);
+        fail_if(begin >= end);
+        fail_if(map.index_back() >= end);
 
         int const size(end - begin);
         int const idx((rand() % size) + begin);
-        ck_assert(idx < map.index_end());
+        fail_if(idx >= map.index_end());
 
         bool size_change;
         how_t how;
@@ -824,9 +821,9 @@ START_TEST(random_test)
 
         if (!Map::not_set(map[idx]))
         {
-            ck_assert_msg((map[idx]%SIZE) == idx, /* see filling loop above */
-                          "Expected %d, got %d %% %d = %d",
-                          idx, map[idx], SIZE, map[idx]%SIZE);
+            fail_if((map[idx]%SIZE) != idx, /* see filling loop above */
+                    "Expected %d, got %d %% %zu = %d",
+                    idx, map[idx], SIZE, map[idx]%SIZE);
         }
 
         map_state const init_state(map);
@@ -836,17 +833,17 @@ START_TEST(random_test)
         case ITERATOR:
             map.erase(map.begin() + (idx - begin));
             if (size_change)
-                ck_assert(init_state != map_state(map));
+                fail_if(init_state == map_state(map));
             else
-                ck_assert(init_state == map_state(map));
+                fail_if(init_state != map_state(map));
 
             break;
         case INDEX:
             map.erase(idx);
             if (size_change)
-                ck_assert(init_state != map_state(map));
+                fail_if(init_state == map_state(map));
             else
-                ck_assert(init_state == map_state(map));
+                fail_if(init_state != map_state(map));
 
             break;
         case PUSHPOP:
@@ -855,7 +852,7 @@ START_TEST(random_test)
             else
                 map.pop_back();
 
-            ck_assert(init_state != map_state(map));
+            fail_if(init_state == map_state(map));
         }
     }
 }
