@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009-2025 Codership Oy <info@codership.com>
+ * Copyright (C) 2009-2020 Codership Oy <info@codership.com>
  */
 
 #include "gcache_bh.hpp"
@@ -8,37 +8,38 @@
 #include <cerrno>
 #include <cassert>
 
+#include <sched.h> // sched_yeild()
+
 namespace gcache
 {
     /*!
      * Reinitialize seqno sequence (after SST or such)
-     * Clears seqno->ptr map // and sets seqno_min to gtid seqno
+     * Clears seqno->ptr map // and sets seqno_min to s.
      */
     void
-    GCache::seqno_reset (const gu::GTID& gtid)
+    GCache::seqno_reset (const gu::UUID& g, seqno_t const s)
     {
         gu::Lock lock(mtx);
 
         assert(seqno2ptr.empty() || seqno_max == seqno2ptr.index_back());
 
-        const seqno_t s(gtid.seqno());
-        if (gtid.uuid() == gid && s != SEQNO_ILL && seqno_max >= s)
+        if (g == gid && s != SEQNO_ILL && seqno_max >= s)
         {
             if (seqno_max > s)
             {
                 discard_tail(s);
                 seqno_max = s;
                 seqno_released = s;
-                assert(seqno2ptr.empty() || seqno_max == seqno2ptr.index_back());
+                assert(seqno_max == seqno2ptr.index_back());
             }
             return;
         }
 
-        log_info << "GCache history reset: " << gu::GTID(gid, seqno_max)
-                 << " -> " << gtid;
+        log_info << "GCache history reset: " << gid << ':' << seqno_max
+                 << " -> " << g << ':' << s;
 
         seqno_released = SEQNO_NONE;
-        gid = gtid.uuid();
+        gid = g;
 
         /* order is significant here */
         rb.seqno_reset();
@@ -54,15 +55,14 @@ namespace gcache
     void
     GCache::seqno_assign (const void* const ptr,
                           seqno_t     const seqno_g,
-                          uint8_t     const type,
-                          bool        const skip)
+                          seqno_t     const seqno_d)
     {
         gu::Lock lock(mtx);
 
         BufferHeader* bh = ptr2BH(ptr);
 
         assert (SEQNO_NONE == bh->seqno_g);
-        assert (seqno_g > 0);
+        assert (SEQNO_ILL  == bh->seqno_d);
         assert (!BH_is_released(bh));
 
         if (gu_likely(seqno_g > seqno_max))
@@ -93,63 +93,7 @@ namespace gcache
         seqno2ptr.insert(seqno_g, ptr);
 
         bh->seqno_g = seqno_g;
-        bh->flags  |= (BUFFER_SKIPPED * skip);
-        bh->type    = type;
-        if (BUFFER_IN_PAGE == bh->store) ps.seqno_assign(bh, seqno_g);
-    }
-
-    /*!
-     * Mark buffer to be skipped
-     */
-    void
-    GCache::seqno_skip (const void* const ptr,
-                        seqno_t     const seqno_g,
-                        uint8_t     const type)
-    {
-        gu::Lock lock(mtx);
-
-        BufferHeader* const bh(ptr2BH(ptr));
-        seqno2ptr_iter_t p = seqno2ptr.find(seqno_g);
-
-        /* sanity checks */
-        int reason(0);
-        std::ostringstream msg;
-
-        if (seqno_g <= 0) {
-            msg << "invalid seqno: " << seqno_g;
-            reason = 1;
-        }
-        else if (seqno_g != bh->seqno_g) {
-            msg << "seqno " << seqno_g << " does not match ptr seqno "
-                << bh->seqno_g;
-            reason = 2;
-        }
-        else if (type != bh->type) {
-            msg << "type " << type << " does not match ptr type "
-                << bh->type;
-            reason = 3;
-        }
-        else if (p == seqno2ptr.end()) {
-            msg << "seqno " << seqno_g << " not found in the map";
-            reason = 4;
-        }
-        else if (ptr != *p) {
-            msg << "ptr " << seqno_g << " does not match mapped ptr "
-                << *p;
-            reason = 5;
-        }
-
-        assert(0 == reason);
-        if (0 != reason)
-        {
-            gu_throw_fatal << "Skipping seqno sanity check failed: " << msg.str()
-                           << " (reason " << reason << ")";
-        }
-
-        assert (!BH_is_released(bh));
-        assert (!BH_is_skipped(bh));
-
-        bh->flags |= BUFFER_SKIPPED;
+        bh->seqno_d = seqno_d;
     }
 
     void
@@ -168,23 +112,16 @@ namespace gcache
         size_t old_gap(-1);
         int    batch_size(min_batch_size);
 
-        bool   loop(true);
+        bool   loop(false);
 
-        while(loop)
+        do
         {
+            /* if we're doing this loop repeatedly, allow other threads to run*/
+            if (loop) sched_yield();
+
             gu::Lock lock(mtx);
 
-#ifndef NDEBUG
-            if (seqno < seqno_released || seqno >= seqno_locked)
-            {
-                if (params.debug())
-                {
-                    log_info << "GCache::seqno_release(" << seqno
-                             << "): seqno_released: " << seqno_released
-                             << ", seqno_locked: " << seqno_locked;
-                }
-            }
-#endif
+            assert(seqno >= seqno_released);
 
             seqno_t idx(seqno2ptr.upper_bound(seqno_released));
 
@@ -209,9 +146,9 @@ namespace gcache
             batch_size += (new_gap >= old_gap) * min_batch_size;
             old_gap = new_gap;
 
-            seqno_t const start  (idx - 1);
-            seqno_t const end    (seqno - start >= 2*batch_size ?
-                                  start + batch_size : seqno);
+            seqno_t const start(idx - 1);
+            seqno_t const end  (seqno - start >= 2*batch_size ?
+                                start + batch_size : seqno);
 #ifndef NDEBUG
             if (params.debug())
             {
@@ -219,7 +156,6 @@ namespace gcache
                          << (seqno - start) << " buffers, batch_size: "
                          << batch_size << ", end: " << end;
             }
-            seqno_t const old_sr(seqno_released);
 #endif
             while((loop = (idx < seqno2ptr.index_end())) && idx <= end)
             {
@@ -231,7 +167,6 @@ namespace gcache
                       seqno_released == SEQNO_NONE))
                 {
                     log_info << "seqno_released: " << seqno_released
-                             << "; seqno_locked: " << seqno_locked
                              << "; idx: " << idx
                              << "; seqno2ptr.begin: " <<seqno2ptr.index_begin()
                              << "\nstart: " << start << "; end: " << end
@@ -250,54 +185,12 @@ namespace gcache
             assert (loop || seqno == seqno_released);
 
             loop = (end < seqno) && loop;
-
-#ifndef NDEBUG
-            if (params.debug())
-            {
-                log_info << "GCache::seqno_release(" << seqno
-                         << ") seqno_released: "
-                         << old_sr << " -> " << seqno_released;
-            }
-#endif
-        }
-    }
-
-    void GCache::seqno_discard(const seqno_t& seqno)
-    {
-#ifndef NDEBUG
-        if (params.debug())
-        {
-            log_info << "GCache::seqno_discard(" << seqno
-                     << ") index_begin: " << seqno2ptr.index_begin();
-        }
-#endif
-
-        /* Gradually discard seqnos in reasonsbly sized batches */
-        while (true)
-        {
-            static int const batch_size(1024);
-            gu::Lock lock(mtx);
-
-            assert(seqno <= seqno_low_);
-
-            if (seqno2ptr.empty()) break;
-
-            seqno_t const s_min(seqno2ptr.index_begin());
-
-            if (s_min > seqno) break;
-
-            // can't discard unassigned seqno
-            assert(seqno_low_ <= seqno2ptr.index_end());
-
-            seqno_t const s_max(std::min(s_min + batch_size, seqno));
-
-            discard_seqno(s_max);
         }
+        while(loop);
     }
 
     /*!
-     * Move lock to a given seqno. Throw gu::NotFound if seqno is not in
-     * cache.
+     * Move lock to a given seqno. Throw gu::NotFound if seqno is not in cache.
      * @throws NotFound
      */
     void GCache::seqno_lock (seqno_t const seqno_g)
@@ -309,58 +202,34 @@ namespace gcache
         assert(SEQNO_MAX == seqno_locked || seqno_locked_count > 0);
         assert(0   == seqno_locked_count || seqno_locked < SEQNO_MAX);
 
-        if (seqno_g <= seqno_low_) throw gu::NotFound();
         seqno2ptr.at(seqno_g); /* check that the element exists */
 
         seqno_locked_count++;
 
-        if (seqno_g < seqno_locked)
-        {
-            seqno_locked = seqno_g;
-            mem.seqno_lock(seqno_locked);
-            rb.seqno_lock(seqno_locked);
-            ps.seqno_lock(seqno_locked);
-        }
+        if (seqno_g < seqno_locked) seqno_locked = seqno_g;
     }
 
     /*!
      * Get pointer to buffer identified by seqno.
-     * Repossesses the buffer if it was already released, preventing its
-     * (and following buffers) discarding.
+     * Moves lock to the given seqno.
      * @throws NotFound
      */
     const void* GCache::seqno_get_ptr (seqno_t const seqno_g,
+                                       seqno_t&      seqno_d,
                                        ssize_t&      size)
     {
-        gu::Lock lock(mtx);
+        const void* ptr;
 
-        const void* const ptr(seqno2ptr.at(seqno_g));
-        assert (ptr);
-
-        BufferHeader* const bh(ptr2BH(ptr));
-        assert(seqno_g == bh->seqno_g);
-
-        if (BH_is_released(bh)) // repossess and revert the effects of free()
         {
-#ifndef NDEBUG
-            buf_tracker.insert(ptr);
-#endif
-            seqno_released = std::min(seqno_released, bh->seqno_g - 1);
-            mallocs++; // to match the resulting frees count
-
-            // notify store
-            switch (bh->store)
-            {
-            case BUFFER_IN_MEM:  mem.repossess(bh); break;
-            case BUFFER_IN_RB:   rb.repossess (bh); break;
-            case BUFFER_IN_PAGE: ps.repossess (bh); break;
-            default: assert(0);
-            }
-
-            bh->flags &= ~BUFFER_RELEASED; // clear released flag
+            gu::Lock lock(mtx);
+            ptr = seqno2ptr.at(seqno_g);
         }
 
-        size = bh->size - sizeof(BufferHeader);
+        assert (ptr);
+
+        const BufferHeader* const bh (ptr2BH(ptr)); // this can result in IO
+        seqno_d = bh->seqno_d;
+        size    = bh->size - sizeof(BufferHeader);
 
         return ptr;
     }
@@ -404,9 +273,8 @@ namespace gcache
             Limits::assert_size(bh->size);
 
             v[i].set_other (bh->seqno_g,
-                            bh->size - sizeof(BufferHeader),
-                            BH_is_skipped(bh),
-                            bh->type);
+                            bh->seqno_d,
+                            bh->size - sizeof(BufferHeader));
         }
 
         return found;
@@ -423,13 +291,7 @@ namespace gcache
         {
             assert(seqno_locked < SEQNO_MAX);
             seqno_locked_count--;
-            if (0 == seqno_locked_count)
-            {
-                seqno_locked = SEQNO_MAX;
-                mem.seqno_unlock();
-                rb.seqno_unlock();
-                ps.seqno_unlock();
-            }
+            if (0 == seqno_locked_count) seqno_locked = SEQNO_MAX;
         }
         else
         {
