@@ -8,38 +8,27 @@
  * See gcs_core.h
  */
 
+#include "gu_throw.hpp"
+
 #define GCS_COMP_MSG_ACCESS
 
 #include "gcs_core.hpp"
 
 #include "gcs_backend.hpp"
 #include "gcs_comp_msg.hpp"
-#include "gcs_code_msg.hpp"
-#include "gcs_error.hpp"
 #include "gcs_fifo_lite.hpp"
 #include "gcs_group.hpp"
 #include "gcs_gcache.hpp"
 
-#include <gu_throw.hpp>
-#include <gu_logger.hpp>
-#include <gu_serialize.hpp>
-#include <gu_debug_sync.hpp>
+#include "gu_debug_sync.hpp"
 
 #include <string.h> // for mempcpy
 #include <errno.h>
 
-#include <cinttypes>
-
-using namespace gcs::core;
-
-void
-gcs_core_register(gu::Config& conf)
+bool
+gcs_core_register (gu_config_t* conf)
 {
-    gcs_group::register_params(conf);
-    if (gcs_backend_register(reinterpret_cast<gu_config_t*>(&conf)))
-    {
-        gu_throw_fatal << "Could not register backend parmeters";
-    }
+    return gcs_backend_register(conf);
 }
 
 const size_t CORE_FIFO_LEN = (1 << 10); // 1024 elements (no need to have more)
@@ -57,21 +46,9 @@ core_state_t;
 
 struct gcs_core
 {
-    gcs_core(gu::Config&  conf,
-             gcache_t*    cache,
-             const char*  node_name,
-             const char*  inc_addr,
-             int          repl_proto_ver,
-             int          appl_proto_ver,
-             int          gcs_proto_ver = GCS_PROTO_MAX);
-    ~gcs_core();
-
     gu_config_t*    config;
     gcache_t*       cache;
 
-    /* group context */
-    gcs_group_t     group;
-
     /* connection per se */
     long            prim_comp_no;
     core_state_t    state;
@@ -91,11 +68,13 @@ struct gcs_core
 
     /* recv part */
     gcs_recv_msg_t  recv_msg;
-    gcs_seqno_t     code_msg_buf;
 
     /* local action FIFO */
     gcs_fifo_lite_t* fifo;
 
+    /* group context */
+    gcs_group_t     group;
+
     /* backend part */
     size_t          msg_size;
     gcs_backend_t   backend;   // message IO context
@@ -118,43 +97,30 @@ core_act_t;
 typedef struct causal_act
 {
     gcs_seqno_t* act_id;
-    gu_uuid_t*   act_uuid;
     long*        error;
     gu_mutex_t*  mtx;
     gu_cond_t*   cond;
 } causal_act_t;
 
-gcs_core::gcs_core(gu::Config&  conf,
-                   gcache_t*    cache,
-                   const char*  node_name,
-                   const char*  inc_addr,
-                   int          repl_proto_ver,
-                   int          appl_proto_ver,
-                   int          gcs_proto_ver)
-    :
-    config(reinterpret_cast<gu_config_t*>(&conf)),
-    cache(cache),
-    group(conf, cache, node_name, inc_addr,
-          gcs_proto_ver, repl_proto_ver,appl_proto_ver),
-    prim_comp_no(),
-    state(),
-    proto_ver(),
-    send_lock(),
-    send_buf(),
-    send_buf_len(),
-    send_act_no(),
-    recv_msg(),
-    code_msg_buf(),
-    fifo(),
-    msg_size(),
-    backend()
-#ifdef GCS_CORE_TESTING
-    ,ls()        // to lock-step in unit tests
-    ,state_uuid()
-#endif
+static int const GCS_PROTO_MAX = 0;
+
+gcs_core_t*
+gcs_core_create (gu_config_t* const conf,
+                 gcache_t*    const cache,
+                 const char*  const node_name,
+                 const char*  const inc_addr,
+                 int          const repl_proto_ver,
+                 int          const appl_proto_ver)
 {
-    auto core(this); // to minimize diff
-    {
+    assert (conf);
+
+    gcs_core_t* core = GU_CALLOC (1, gcs_core_t);
+
+    if (NULL != core) {
+
+        core->config = conf;
+        core->cache  = cache;
+
         // Need to allocate something, otherwise Spread 3.17.3 freaks out.
         core->recv_msg.buf = gu_malloc(CORE_INIT_BUF_SIZE);
         if (core->recv_msg.buf) {
@@ -170,15 +136,17 @@ gcs_core::gcs_core(gu::Config&  conf,
                                                    sizeof (core_act_t));
                 if (core->fifo) {
                     gu_mutex_init  (&core->send_lock, NULL);
-                    core->proto_ver = -1;
-                    // ^^^ shall be bumped in gcs_group_act_conf()
+                    core->proto_ver = -1; // shall be bumped in gcs_group_act_conf()
+                    gcs_group_init (&core->group, cache, node_name, inc_addr,
+                                    GCS_PROTO_MAX, repl_proto_ver,
+                                    appl_proto_ver);
                     core->state = CORE_CLOSED;
                     core->send_act_no = 1; // 0 == no actions sent
 #ifdef GCS_CORE_TESTING
                     gu_lock_step_init (&core->ls);
                     core->state_uuid = GU_UUID_NIL;
 #endif
-                    return; // success
+                    return core; // success
                 }
 
                 gu_free (core->send_buf);
@@ -186,34 +154,18 @@ gcs_core::gcs_core(gu::Config&  conf,
 
             gu_free (core->recv_msg.buf);
         }
+
+        gu_free (core);
     }
 
-    gu_throw_fatal << "Failed to initialize GCS core";
-}
-
-gcs_core_t*
-gcs_core_create (gu::Config&  conf,
-                 gcache_t*    const cache,
-                 const char*  const node_name,
-                 const char*  const inc_addr,
-                 int          const repl_proto_ver,
-                 int          const appl_proto_ver,
-                 int          const gcs_proto_ver)
-{
-    try {
-        return new gcs_core(conf, cache, node_name, inc_addr,
-                            repl_proto_ver, appl_proto_ver, gcs_proto_ver);
-    }
-    catch (...) {
-        return nullptr;
-    }
+    return NULL; // failure
 }
 
 long
-gcs_core_init (gcs_core_t* core, const gu::GTID& position)
+gcs_core_init (gcs_core_t* core, gcs_seqno_t seqno, const gu_uuid_t* uuid)
 {
     if (core->state == CORE_CLOSED) {
-        return gcs_group_init_history (&core->group, position);
+        return gcs_group_init_history (&core->group, seqno, uuid);
     }
     else {
         gu_error ("State must be CLOSED");
@@ -253,14 +205,14 @@ gcs_core_open (gcs_core_t* core,
             core->state = CORE_NON_PRIMARY;
         }
         else {
-            gu_error ("Failed to open backend connection: %ld (%s)",
+            gu_error ("Failed to open backend connection: %d (%s)",
                       ret, strerror(-ret));
             core->backend.destroy (&core->backend);
         }
 
     }
     else {
-        gu_error ("Failed to initialize backend using '%s': %ld (%s)",
+        gu_error ("Failed to initialize backend using '%s': %d (%s)",
                   url, ret, strerror(-ret));
     }
 
@@ -388,7 +340,7 @@ gcs_core_send (gcs_core_t*          const conn,
     }
     else {
         ret = core_error (conn->state);
-        gu_error ("Failed to access core FIFO: %zd (%s)", ret, strerror (-ret));
+        gu_error ("Failed to access core FIFO: %d (%s)", ret, strerror (-ret));
         return ret;
     }
 
@@ -516,14 +468,12 @@ core_msg_recv (gcs_backend_t* backend, gcs_recv_msg_t* recv_msg,
 
     ret = backend->recv (backend, recv_msg, timeout);
 
-    assert(recv_msg->buf || 0 == recv_msg->buf_len);
-
     while (gu_unlikely(ret > recv_msg->buf_len)) {
         /* recv_buf too small, reallocate */
         /* sometimes - like in case of component message, we may need to
          * do reallocation 2 times. This should be fixed in backend */
         void* msg = gu_realloc (recv_msg->buf, ret);
-        gu_debug ("Reallocating buffer from %d to %ld bytes",
+        gu_debug ("Reallocating buffer from %d to %d bytes",
                   recv_msg->buf_len, ret);
         if (msg) {
             /* try again */
@@ -537,16 +487,14 @@ core_msg_recv (gcs_backend_t* backend, gcs_recv_msg_t* recv_msg,
         }
         else {
             /* realloc unsuccessfull, old recv_buf remains */
-            gu_error ("Failed to reallocate buffer to %ld bytes", ret);
+            gu_error ("Failed to reallocate buffer to %d bytes", ret);
             ret = -ENOMEM;
             break;
         }
     }
 
-    assert(recv_msg->buf);
-
     if (gu_unlikely(ret < 0)) {
-        gu_debug ("returning %ld: %s\n", ret, strerror(-ret));
+        gu_debug ("returning %d: %s\n", ret, strerror(-ret));
     }
 
     return ret;
@@ -573,10 +521,10 @@ core_handle_act_msg (gcs_core_t*          core,
     if ((CORE_PRIMARY == core->state) || my_msg){//should always handle own msgs
 
         if (gu_unlikely(gcs_act_proto_ver(msg->buf) !=
-                        gcs_core_proto_ver(core))) {
-            gu_info ("Message with protocol version %d != highest commonly "
-                     "supported: %d.",
-                     gcs_act_proto_ver(msg->buf), gcs_core_proto_ver(core));
+                        gcs_core_group_protocol_version(core))) {
+            gu_info ("Message with protocol version %d != highest commonly supported: %d. ",
+                     gcs_act_proto_ver(msg->buf),
+                     gcs_core_group_protocol_version(core));
             commonly_supported_version = false;
             if (!my_msg) {
                 gu_info ("Discard message from member %d because of "
@@ -607,11 +555,11 @@ core_handle_act_msg (gcs_core_t*          core,
 #else
             assert (NULL == act->act.buf);
 #endif
-            assert(act->sender_idx == msg->sender_idx);
+            act->sender_idx = msg->sender_idx;
 
             if (gu_likely(!my_msg)) {
                 /* foreign action, must be passed from gcs_group */
-                assert (GCS_ACT_WRITESET != act->act.type || act->id > 0);
+                assert (GCS_ACT_TORDERED != act->act.type || act->id > 0);
             }
             else {
                 /* local action, get from FIFO, should be there already */
@@ -630,10 +578,8 @@ core_handle_act_msg (gcs_core_t*          core,
                     /* NOTE! local_act cannot be used after this point */
                     /* sanity check */
                     if (gu_unlikely(sent_act_id != frg.act_id)) {
-                        gu_fatal("FIFO violation: expected sent_act_id %" PRId64
-                                 " "
-                                 "found %" PRId64,
-                                 sent_act_id, frg.act_id);
+                        gu_fatal ("FIFO violation: expected sent_act_id %lld "
+                                  "found %lld", sent_act_id, frg.act_id);
                         ret = -ENOTRECOVERABLE;
                     }
                     if (gu_unlikely(act->act.buf_len != ret)) {
@@ -676,7 +622,7 @@ core_handle_act_msg (gcs_core_t*          core,
                 ret = gcs_group_handle_state_request (group, act);
                 assert (ret <= 0 || ret == act->act.buf_len);
 #ifdef GCS_FOR_GARB
-                if (ret < 0) gu_fatal ("Handling state request failed: %ld",ret);
+                if (ret < 0) gu_fatal ("Handling state request failed: %d",ret);
                 act->act.buf = NULL;
             }
             else {
@@ -700,7 +646,7 @@ core_handle_act_msg (gcs_core_t*          core,
     }
     else {
         /* Non-primary conf, foreign message - ignore */
-        gu_info ("Action message in non-primary configuration from "
+        gu_warn ("Action message in non-primary configuration from "
                  "member %d", msg->sender_idx);
         ret = 0;
     }
@@ -725,28 +671,19 @@ core_handle_last_msg (gcs_core_t*          core,
                       struct gcs_recv_msg* msg,
                       struct gcs_act*      act)
 {
-    assert(GCS_MSG_LAST == msg->type);
-    assert(CodeMsg::serial_size() >= msg->size);
-    assert(int(sizeof(uint64_t)) <= msg->size);
+    assert (GCS_MSG_LAST == msg->type);
 
-    if (gu_likely(gcs_group_is_primary(&core->group))) {
-
-        gcs_seqno_t const commit_cut
-            (gcs_group_handle_last_msg(&core->group, msg));
-
-        if (0 != commit_cut) {
+    if (gcs_group_is_primary(&core->group)) {
+        gcs_seqno_t commit_cut =
+            gcs_group_handle_last_msg (&core->group, msg);
+        if (commit_cut) {
             /* commit cut changed */
-            int   const buf_len(sizeof(uint64_t));
-            void* const buf(malloc(buf_len));
-
-            if (gu_likely(NULL != (buf))) {
+            if ((act->buf = malloc (sizeof (commit_cut)))) {
+                act->type                 = GCS_ACT_COMMIT_CUT;
                 /* #701 - everything that goes into the action buffer
                  *        is expected to be serialized. */
-                gu::serialize8(commit_cut, buf, buf_len, 0);
-                assert(NULL == act->buf);
-                act->buf     = buf;
-                act->buf_len = buf_len;
-                act->type    = GCS_ACT_COMMIT_CUT;
+                *((gcs_seqno_t*)act->buf) = gcs_seqno_htog(commit_cut);
+                act->buf_len              = sizeof(commit_cut);
                 return act->buf_len;
             }
             else {
@@ -763,46 +700,6 @@ core_handle_last_msg (gcs_core_t*          core,
     return 0;
 }
 
-/*!
- * Helper for gcs_core_recv(). Handles GCS_MSG_LAST.
- *
- * @return action size, negative error code or 0 to continue.
- */
-static int
-core_handle_vote_msg (gcs_core_t*          core,
-                      struct gcs_recv_msg* msg,
-                      struct gcs_act*      act)
-{
-    assert (GCS_MSG_VOTE == msg->type);
-    assert (CodeMsg::serial_size() <= msg->size);
-
-    VoteResult const res(gcs_group_handle_vote_msg(&core->group, msg));
-
-    if (res.seqno != GCS_SEQNO_ILL)
-    {
-        assert(res.seqno > 0);
-        /* voting complete or vote request */
-        int   const buf_len(2 * sizeof(uint64_t));
-        void* const buf(malloc(buf_len));
-
-        if (gu_likely(NULL != (buf))) {
-            gu::serialize8(res.seqno, buf, buf_len, 0);
-            gu::serialize8(res.res,   buf, buf_len, 8);
-            assert(NULL == act->buf);
-            act->buf     = buf;
-            act->buf_len = buf_len;
-            act->type    = GCS_ACT_VOTE;
-            return act->buf_len;
-        }
-        else {
-            gu_fatal ("Out of memory for GCS_ACT_VOTE");
-            return -ENOMEM;
-        }
-    }
-
-    return 0;
-}
-
 /*! Common things to do on detected inconsistency */
 static int
 core_handle_inconsistency(gcs_core_t* core, struct gcs_act* act)
@@ -820,18 +717,17 @@ core_handle_inconsistency(gcs_core_t* core, struct gcs_act* act)
  * @return action size, negative error code or 0 to continue.
  */
 static ssize_t
-core_handle_comp_msg (gcs_core_t*          const core,
-                      struct gcs_recv_msg* const msg,
-                      struct gcs_act_rcvd* const rcvd)
+core_handle_comp_msg (gcs_core_t*          core,
+                      struct gcs_recv_msg* msg,
+                      struct gcs_act*      act)
 {
-    ssize_t ret(0);
-    gcs_group_t*    const group(&core->group);
-    struct gcs_act* const act(&rcvd->act);
+    ssize_t      ret = 0;
+    gcs_group_t* group = &core->group;
 
     assert (GCS_MSG_COMPONENT == msg->type);
 
     if (msg->size < (ssize_t)sizeof(gcs_comp_msg_t)) {
-        gu_error ("Malformed component message (size %d < %zu). Ignoring",
+        gu_error ("Malformed component message (size %zd < %zd). Ignoring",
                   msg->size, sizeof(gcs_comp_msg_t));
         return 0;
     }
@@ -848,9 +744,9 @@ core_handle_comp_msg (gcs_core_t*          const core,
         assert (CORE_EXCHANGE != core->state);
         if (CORE_NON_PRIMARY == core->state) core->state = CORE_PRIMARY;
 
-        ret = gcs_group_act_conf (group, rcvd, &core->proto_ver);
+        ret = gcs_group_act_conf (group, act, &core->proto_ver);
         if (ret < 0) {
-            gu_fatal ("Failed create PRIM CONF action: %zd (%s)",
+            gu_fatal ("Failed create PRIM CONF action: %d (%s)",
                       ret, strerror (-ret));
             assert (0);
             ret = -ENOTRECOVERABLE;
@@ -873,25 +769,11 @@ core_handle_comp_msg (gcs_core_t*          const core,
                                           &uuid,
                                           sizeof(uuid),
                                           GCS_MSG_STATE_UUID);
-                if (ret < 0)
-                {
+                if (ret < 0) {
                     // if send() failed, it means new configuration change
                     // is on the way. Probably should ignore.
-                    switch (-ret)
-                    {
-                    case EAGAIN:
-                        gu_info("Temporary failure in sending state UUID, "
-                                "will try again in next primary component");
-                        break;
-                    case ENOTCONN:
-                        gu_info("Failed to send state UUID: Connection to "
-                                "cluster was closed");
-                        break;
-                    default:
-                        gu_warn("Failed to send state UUID: %zd (%s)", ret,
-                                gcs_error_str(-ret));
-                        break;
-                    }
+                    gu_warn ("Failed to send state UUID: %d (%s)",
+                             ret, strerror (-ret));
                 }
                 else {
                     gu_info ("STATE_EXCHANGE: sent state UUID: "
@@ -908,30 +790,30 @@ core_handle_comp_msg (gcs_core_t*          const core,
     case GCS_GROUP_NON_PRIMARY:
         /* Lost primary component */
         if (core->state < CORE_CLOSED) {
+            ret = gcs_group_act_conf (group, act, &core->proto_ver);
+            if (ret < 0) {
+                gu_fatal ("Failed create NON-PRIM CONF action: %d (%s)",
+                          ret, strerror (-ret));
+                assert (0);
+                ret = -ENOTRECOVERABLE;
+            }
+
             if (gcs_group_my_idx(group) == -1) { // self-leave
                 gcs_fifo_lite_close (core->fifo);
                 core->state = CORE_CLOSED;
-                ret = -gcs_comp_msg_error((const gcs_comp_msg_t*)msg->buf);
-                if (ret < 0) {
-                    assert(act->buf == NULL);
-                    assert(act->buf_len == 0);
+                if (gcs_comp_msg_error((const gcs_comp_msg_t*)msg->buf)) {
+                    ret = -gcs_comp_msg_error(
+                        (const gcs_comp_msg_t*)msg->buf);
+                    free(const_cast<void*>(act->buf));
+                    act->buf = NULL;
+                    act->buf_len = 0;
                     act->type = GCS_ACT_ERROR;
-                    gu_debug("comp msg error in core %ld", -ret);
+                    gu_info("comp msg error in core %d", -ret);
                 }
             }
             else {                               // regular non-prim
                 core->state = CORE_NON_PRIMARY;
             }
-
-            if (GCS_GROUP_NON_PRIMARY == ret) { // no error in comp msg
-                ret = gcs_group_act_conf (group, rcvd, &core->proto_ver);
-                if (ret < 0) {
-                    gu_fatal ("Failed create NON-PRIM CONF action: %ld (%s)",
-                              ret, strerror (-ret));
-                    assert (0);
-                    ret = -ENOTRECOVERABLE;
-                }
-            }
         }
         else { // ignore in production?
             assert(0);
@@ -948,7 +830,7 @@ core_handle_comp_msg (gcs_core_t*          const core,
         assert(0);
         // fall through
     default:
-        gu_fatal ("Failed to handle component message: %ld (%s)!",
+        gu_fatal ("Failed to handle component message: %d (%s)!",
                   ret, strerror (-ret));
         assert(0);
     }
@@ -999,7 +881,7 @@ core_handle_uuid_msg (gcs_core_t*     core,
                         // This may happen if new configuraiton chage goes on.
                         // What shall we do in this case? Is it unrecoverable?
                         gu_error ("STATE EXCHANGE: failed for: " GU_UUID_FORMAT
-                                 ": %zd (%s)",
+                                 ": %d (%s)",
                                  GU_UUID_ARGS(state_uuid), ret, strerror(-ret));
                     }
                     gcs_state_msg_destroy (state);
@@ -1015,7 +897,7 @@ core_handle_uuid_msg (gcs_core_t*     core,
             break;
         default:
             assert(ret < 0);
-            gu_error ("Failed to handle state UUID: %zd (%s)",
+            gu_error ("Failed to handle state UUID: %d (%s)",
                       ret, strerror (-ret));
         }
     }
@@ -1031,19 +913,17 @@ core_handle_uuid_msg (gcs_core_t*     core,
 static ssize_t
 core_handle_state_msg (gcs_core_t*          core,
                        struct gcs_recv_msg* msg,
-                       struct gcs_act_rcvd* rcvd)
+                       struct gcs_act*      act)
 {
-    ssize_t      ret(0);
-    gcs_group_t* const group(&core->group);
+    ssize_t      ret = 0;
+    gcs_group_t* group = &core->group;
 
     assert (GCS_MSG_STATE_MSG == msg->type);
 
     if (GCS_GROUP_WAIT_STATE_MSG == gcs_group_state (group))
     {
         if (gu_mutex_lock (&core->send_lock)) abort();
-        // cast to int is needed to distinguish between positive enums and
-        // negative error codes
-        ret = int(gcs_group_handle_state_msg (group, msg));
+        ret = gcs_group_handle_state_msg (group, msg);
 
         switch (ret) {
         case GCS_GROUP_PRIMARY:
@@ -1064,25 +944,25 @@ core_handle_state_msg (gcs_core_t*          core,
                 }
             }
 
-            ret = gcs_group_act_conf (group, rcvd, &core->proto_ver);
+            ret = gcs_group_act_conf (group, act, &core->proto_ver);
             if (ret < 0) {
-                gu_fatal ("Failed create CONF action: %zd (%s)",
+                gu_fatal ("Failed create CONF action: %d (%s)",
                           ret, strerror (-ret));
                 assert (0);
                 ret = -ENOTRECOVERABLE;
             }
-            assert (ret == rcvd->act.buf_len);
+            assert (ret == act->buf_len);
             break;
         case GCS_GROUP_WAIT_STATE_MSG:
             // waiting for more state messages
             ret = 0;
             break;
         case GCS_GROUP_INCONSISTENT:
-            ret = core_handle_inconsistency(core, &rcvd->act);
+            ret = core_handle_inconsistency(core, act);
             break;
         default:
             assert (ret < 0);
-            gu_error ("Failed to handle state message: %zd (%s)",
+            gu_error ("Failed to handle state message: %d (%s)",
                       ret, strerror (-ret));
         }
         gu_mutex_unlock (&core->send_lock);
@@ -1091,30 +971,6 @@ core_handle_state_msg (gcs_core_t*          core,
     return ret;
 }
 
-/* returns code in serialized form */
-static gcs_seqno_t
-core_msg_code (const struct gcs_recv_msg* const msg, int const proto_ver)
-{
-    if (gu_likely(proto_ver >= 1 &&
-                  msg->size == gcs::core::CodeMsg::serial_size()))
-    {
-        const gcs::core::CodeMsg* const cm
-            (static_cast<const gcs::core::CodeMsg*>(msg->buf));
-        return gu::htog(cm->code());
-    }
-    else if (proto_ver == 0 && msg->size == sizeof(gcs_seqno_t))
-    {
-        return *(static_cast<const gcs_seqno_t*>(msg->buf));
-        // no deserialization
-    }
-    else
-    {
-        log_warn << "Bogus code message size: " << msg->size;
-        assert(0);
-        return gu::htog(gcs_seqno_t(-EINVAL));
-    }
-}
-
 /*!
  * Some service actions are for internal use and consist of a single message
  * (FLOW, JOIN, SYNC)
@@ -1122,22 +978,21 @@ core_msg_code (const struct gcs_recv_msg* const msg, int const proto_ver)
  * can guarantee that we don't deallocate it. Action here is just a wrapper
  * to deliver message to the upper level.
  */
-static int
+static ssize_t
 core_msg_to_action (gcs_core_t*          core,
                     struct gcs_recv_msg* msg,
                     struct gcs_act_rcvd* rcvd)
 {
-    int          ret = 0;
+    ssize_t      ret = 0;
     gcs_group_t* group = &core->group;
-    struct gcs_act* const act(&rcvd->act);
 
     if (GCS_GROUP_PRIMARY == gcs_group_state (group)) {
+        gcs_act_type_t act_type;
+
         switch (msg->type) {
         case GCS_MSG_FLOW: // most frequent
             ret = 1;
-            act->type    = GCS_ACT_FLOW;
-            act->buf     = msg->buf;
-            act->buf_len = msg->size;
+            act_type = GCS_ACT_FLOW;
             break;
         case GCS_MSG_JOIN:
             ret = gcs_group_handle_join_msg (group, msg);
@@ -1151,26 +1006,14 @@ core_msg_to_action (gcs_core_t*          core,
                 // so this must be done here.
                 gu_abort();
             }
-            else if (ret != 0)
-            {
-                core->code_msg_buf = core_msg_code(msg, core->proto_ver);
-                act->type    = GCS_ACT_JOIN;
-                act->buf     = &core->code_msg_buf;
-                act->buf_len = sizeof(core->code_msg_buf);
-            }
+            act_type = GCS_ACT_JOIN;
             break;
         case GCS_MSG_SYNC:
             ret = gcs_group_handle_sync_msg (group, msg);
-            if (gu_likely(ret != 0))
-            {
-                core->code_msg_buf = core_msg_code(msg, core->proto_ver);
-                act->type    = GCS_ACT_SYNC;
-                act->buf     = &core->code_msg_buf;
-                act->buf_len = sizeof(core->code_msg_buf);
-            }
+            act_type = GCS_ACT_SYNC;
             break;
         default:
-            gu_error ("Iternal error. Unexpected message type %s from %d",
+            gu_error ("Iternal error. Unexpected message type %s from %ld",
                       gcs_msg_type_string[msg->type], msg->sender_idx);
             assert (0);
             ret = -EPROTO;
@@ -1180,14 +1023,15 @@ core_msg_to_action (gcs_core_t*          core,
             if      (ret > 0) rcvd->id = 0;
             else if (ret < 0) rcvd->id = ret;
 
-            ret = act->buf_len;
+            struct gcs_act* const act(&rcvd->act);
+            act->type    = act_type;
+            act->buf     = msg->buf;
+            act->buf_len = msg->size;
+            ret          = msg->size;
         }
     }
     else {
-        /* Messages which were sent just before cluster partitioning may
-         * be delivered in the following non-primary configuration. This
-         * is expected behavior, so info log level is enough. */
-        gu_info ("%s message from member %d in non-primary configuration. "
+        gu_warn ("%s message from member %ld in non-primary configuration. "
                  "Ignored.", gcs_msg_type_string[msg->type], msg->sender_idx);
     }
 
@@ -1199,19 +1043,18 @@ static long core_msg_causal(gcs_core_t* conn,
 {
     if (gu_unlikely(msg->size != sizeof(causal_act_t)))
     {
-        gu_error("invalid causal act len %d, expected %zu",
+        gu_error("invalid causal act len %ld, expected %ld",
                  msg->size, sizeof(causal_act_t));
         return -EPROTO;
     }
 
-    causal_act_t* act = (causal_act_t*)msg->buf;
+    causal_act_t* act= (causal_act_t*)msg->buf;
     gu_mutex_lock(act->mtx);
     {
         switch (conn->group.state)
         {
         case GCS_GROUP_PRIMARY:
             *act->act_id = conn->group.act_id_;
-            *act->act_uuid = conn->group.group_uuid;
             break;
         case GCS_GROUP_WAIT_STATE_UUID:
         case GCS_GROUP_WAIT_STATE_MSG:
@@ -1233,15 +1076,15 @@ ssize_t gcs_core_recv (gcs_core_t*          conn,
                        struct gcs_act_rcvd* recv_act,
                        long long            timeout)
 {
-    struct gcs_recv_msg* const recv_msg(&conn->recv_msg);
-    ssize_t ret(0);
+    struct gcs_recv_msg* recv_msg = &conn->recv_msg;
+    ssize_t              ret      = 0;
 
     static struct gcs_act_rcvd zero_act(
         gcs_act(NULL,
                 0,
                 GCS_ACT_ERROR),
         NULL,
-        GCS_SEQNO_ILL,
+        -1,   // GCS_SEQNO_ILL
         -1);
 
     *recv_act = zero_act;
@@ -1261,9 +1104,6 @@ ssize_t gcs_core_recv (gcs_core_t*          conn,
             goto out; /* backend error while receiving message */
         }
 
-        assert(recv_msg->buf);
-        assert(recv_msg->buf_len >= recv_msg->size);
-
         switch (recv_msg->type) {
         case GCS_MSG_ACTION:
             ret = core_handle_act_msg(conn, recv_msg, recv_act);
@@ -1275,7 +1115,7 @@ ssize_t gcs_core_recv (gcs_core_t*          conn,
             assert (ret == recv_act->act.buf_len);
             break;
         case GCS_MSG_COMPONENT:
-            ret = core_handle_comp_msg (conn, recv_msg, recv_act);
+            ret = core_handle_comp_msg (conn, recv_msg, &recv_act->act);
             // assert (ret >= 0); // hang on error in debug mode
             assert (ret == recv_act->act.buf_len || ret < 0);
             break;
@@ -1285,8 +1125,8 @@ ssize_t gcs_core_recv (gcs_core_t*          conn,
             ret = 0;              // continue waiting for state messages
             break;
         case GCS_MSG_STATE_MSG:
-            ret = core_handle_state_msg (conn, recv_msg, recv_act);
-            // assert (ret >= 0); // hang on error in debug mode
+            ret = core_handle_state_msg (conn, recv_msg, &recv_act->act);
+            //assert (ret >= 0); // hang on error in debug mode
             assert (ret == recv_act->act.buf_len || ret < 0);
             break;
         case GCS_MSG_JOIN:
@@ -1295,11 +1135,6 @@ ssize_t gcs_core_recv (gcs_core_t*          conn,
             ret = core_msg_to_action (conn, recv_msg, recv_act);
             assert (ret == recv_act->act.buf_len || ret <= 0);
             break;
-        case GCS_MSG_VOTE:
-            ret = core_handle_vote_msg(conn, recv_msg, &recv_act->act);
-            assert (ret >= 0); // hang on error in debug mode
-            assert (ret == recv_act->act.buf_len);
-            break;
         case GCS_MSG_CAUSAL:
             ret = core_msg_causal(conn, recv_msg);
             assert(recv_msg->sender_idx == gcs_group_my_idx(&conn->group));
@@ -1320,20 +1155,18 @@ out:
 
     assert (ret || GCS_ACT_ERROR == recv_act->act.type);
     assert (ret == recv_act->act.buf_len || ret < 0);
-    assert (recv_act->id       <= 0                ||
-            recv_act->act.type == GCS_ACT_WRITESET ||
-            recv_act->act.type == GCS_ACT_CCHANGE  ||
+    assert (recv_act->id       <= 0 ||
+            recv_act->act.type == GCS_ACT_TORDERED ||
             recv_act->act.type == GCS_ACT_STATE_REQ); // <- dirty hack
     assert (recv_act->sender_idx >= 0 ||
-            recv_act->act.type   != GCS_ACT_WRITESET);
+            recv_act->act.type   != GCS_ACT_TORDERED);
 
 //    gu_debug ("Returning %d", ret);
 
-    if (gu_unlikely(ret < 0)) {
+    if (ret < 0) {
         assert (recv_act->id < 0);
-        assert (GCS_ACT_CCHANGE != recv_act->act.type);
 
-        if (GCS_ACT_WRITESET == recv_act->act.type && recv_act->act.buf) {
+        if (GCS_ACT_TORDERED == recv_act->act.type && recv_act->act.buf) {
             gcs_gcache_free (conn->cache, recv_act->act.buf);
             recv_act->act.buf = NULL;
         }
@@ -1342,8 +1175,6 @@ out:
             conn->backend.close(&conn->backend);
             if (GCS_ACT_INCONSISTENCY != recv_act->act.type) {
                 /* inconsistency event must be passed up */
-                gu_fatal("Unrecoverable error happened above. Aborting...");
-                usleep(1000000); // give it a second
                 gu_abort();
             }
         }
@@ -1371,11 +1202,12 @@ long gcs_core_close (gcs_core_t* core)
     return ret;
 }
 
-static int
-core_destroy(gcs_core_t* core)
+long gcs_core_destroy (gcs_core_t* core)
 {
     core_act_t* tmp;
 
+    if (!core) return -EBADFD;
+
     if (gu_mutex_lock (&core->send_lock)) return -EBADFD;
     {
         if (CORE_CLOSED != core->state) {
@@ -1403,6 +1235,7 @@ core_destroy(gcs_core_t* core)
         gcs_fifo_lite_pop_head (core->fifo);
     }
     gcs_fifo_lite_destroy (core->fifo);
+    gcs_group_free (&core->group);
 
     /* free buffers */
     gu_free (core->recv_msg.buf);
@@ -1412,30 +1245,13 @@ core_destroy(gcs_core_t* core)
     gu_lock_step_destroy (&core->ls);
 #endif
 
+    gu_free (core);
+
     return 0;
 }
 
-gcs_core::~gcs_core()
-{
-    int const ret(core_destroy(this));
-    if (ret) {
-        gu_throw_error(ret) << "GCS core destructor failed";
-    }
-}
-
-long gcs_core_destroy (gcs_core_t* core)
-{
-    try {
-        delete core;
-        return 0;
-    }
-    catch (...) {
-        return -1;
-    }
-}
-
-int
-gcs_core_proto_ver (const gcs_core_t* conn)
+gcs_proto_t
+gcs_core_group_protocol_version (const gcs_core_t* conn)
 {
     return conn->proto_ver;
 }
@@ -1496,89 +1312,41 @@ gcs_core_set_pkt_size (gcs_core_t* core, int const pkt_size)
     return ret;
 }
 
-static inline ssize_t
+static inline long
 core_send_seqno (gcs_core_t* core, gcs_seqno_t seqno, gcs_msg_type_t msg_type)
 {
     gcs_seqno_t const htogs = gcs_seqno_htog (seqno);
     ssize_t           ret   = core_msg_send_retry (core, &htogs,
                                                    sizeof(htogs),
                                                    msg_type);
-    if (ret > 0) { assert(ret == sizeof(seqno)); }
-
-    return ret;
-}
-
-static inline int
-core_send_code (gcs_core_t* const core, const gu::GTID& gtid, int64_t code,
-                gcs_msg_type_t const msg_type)
-{
-    if (gu_unlikely(core->proto_ver < 1))
-    {
-        return core_send_seqno (core, code < 0 ? code : gtid.seqno(), msg_type);
+    if (ret > 0) {
+        assert(ret == sizeof(seqno));
+        ret = 0;
     }
 
-    CodeMsg const msg(gtid, code);
-    assert(msg.uuid() != GU_UUID_NIL);
-
-    int ret(core_msg_send_retry (core, msg(), msg.serial_size(), msg_type));
-
-    if (ret > 0) { assert(ret == msg.serial_size()); }
-
     return ret;
 }
 
-int
-gcs_core_set_last_applied (gcs_core_t* const core, const gu::GTID& gtid)
+long
+gcs_core_set_last_applied (gcs_core_t* core, gcs_seqno_t seqno)
 {
-    return core_send_code (core, gtid, 0, GCS_MSG_LAST);
+    return core_send_seqno (core, seqno, GCS_MSG_LAST);
 }
 
-int
-gcs_core_send_join (gcs_core_t* const core, const gu::GTID& gtid, int code)
+long
+gcs_core_send_join (gcs_core_t* core, gcs_seqno_t seqno)
 {
-    return core_send_code (core, gtid, code, GCS_MSG_JOIN);
+    return core_send_seqno (core, seqno, GCS_MSG_JOIN);
 }
 
-int
-gcs_core_send_sync (gcs_core_t* const core, const gu::GTID& gtid)
+long
+gcs_core_send_sync (gcs_core_t* core, gcs_seqno_t seqno)
 {
-    return core_send_code (core, gtid, 0, GCS_MSG_SYNC);
+    return core_send_seqno (core, seqno, GCS_MSG_SYNC);
 }
 
-int
-gcs_core_send_vote (gcs_core_t* const core, const gu::GTID& gtid, int64_t code,
-                    const void* data, size_t const data_len)
-{
-#if 0 // simple code message
-    return core_send_code (core, gtid, code, GCS_MSG_VOTE);
-#else
-    CodeMsg const cmsg(gtid, code);
-    assert(cmsg.uuid() != GU_UUID_NIL);
-    int const cmsg_size(cmsg.serial_size());
-
-    char vmsg[1024] = { 0, }; // try to fit in one ethernet frame
-    assert(cmsg_size < int(sizeof(vmsg)));
-
-    ::memcpy(&vmsg[0], cmsg(), cmsg_size);
-
-    int copy_size(int(sizeof(vmsg)) - cmsg_size - 1); // allow for trailing 0
-    assert(copy_size >= 0);
-    if (size_t(copy_size) > data_len) copy_size = data_len;
-
-    ::memcpy(&vmsg[cmsg_size], data, copy_size);
-
-    int const vmsg_size(cmsg_size + copy_size + 1);
-
-    int ret(core_msg_send_retry(core, &vmsg[0], vmsg_size, GCS_MSG_VOTE));
-
-    if (ret > 0) { assert(ret >= cmsg_size); }
-
-    return ret;
-#endif
-}
-
-ssize_t
-gcs_core_send_fc (gcs_core_t* core, const void* const fc, size_t const fc_size)
+long
+gcs_core_send_fc (gcs_core_t* core, const void* fc, size_t fc_size)
 {
     ssize_t ret;
     ret = core_msg_send_retry (core, fc, fc_size, GCS_MSG_FLOW);
@@ -1589,14 +1357,12 @@ gcs_core_send_fc (gcs_core_t* core, const void* const fc, size_t const fc_size)
 }
 
 long
-gcs_core_caused (gcs_core_t* core, gu::GTID& gtid)
+gcs_core_caused (gcs_core_t* core, gcs_seqno_t& seqno)
 {
     long         error = 0;
-    gcs_seqno_t  act_id = GCS_SEQNO_ILL;
-    gu_uuid_t    act_uuid = GU_UUID_NIL;
     gu_mutex_t   mtx;
     gu_cond_t    cond;
-    causal_act_t act = {&act_id, &act_uuid, &error, &mtx, &cond};
+    causal_act_t act = {&seqno, &error, &mtx, &cond};
 
     gu_mutex_init (&mtx, NULL);
     gu_cond_init  (&cond, NULL);
@@ -1610,10 +1376,6 @@ gcs_core_caused (gcs_core_t* core, gu::GTID& gtid)
         if (ret == sizeof(act))
         {
             gu_cond_wait (&cond, &mtx);
-            if (error == 0)
-            {
-              gtid.set (act_uuid, act_id);
-            }
         }
         else
         {
@@ -1628,13 +1390,11 @@ gcs_core_caused (gcs_core_t* core, gu::GTID& gtid)
     return error;
 }
 
-int
+long
 gcs_core_param_set (gcs_core_t* core, const char* key, const char* value)
 {
     if (core->backend.conn) {
-        return
-            gcs_group_param_set(core->group, key, value) &&
-            core->backend.param_set(&core->backend, key, value);
+        return core->backend.param_set (&core->backend, key, value);
     }
     else {
         return 1;
@@ -1664,11 +1424,6 @@ void gcs_core_get_status(gcs_core_t* core, gu::Status& status)
     gu_mutex_unlock(&core->send_lock);
 }
 
-void gcs_core_get_protocols(gcs_core_t* core, int& appl, int& repl, int& gcs)
-{
-    core->group.get_protocols(appl, repl, gcs);
-}
-
 #ifdef GCS_CORE_TESTING
 
 gcs_backend_t*
@@ -1706,5 +1461,4 @@ gcs_core_get_fifo (gcs_core_t* core)
 {
     return core->fifo;
 }
-
 #endif /* GCS_CORE_TESTING */
