@@ -1,14 +1,18 @@
 /*
- * Copyright (C) 2008-2020 Codership Oy <info@codership.com>
+ * Copyright (C) 2008-2014 Codership Oy <info@codership.com>
  *
  * $Id$
  */
 
+#include <check.h>
+#include <errno.h>
+#include <string.h>
+#include <stdio.h>
+#include <unistd.h>
 #include <galerautils.h>
 
 #include "../gcs_backend.hpp"
-
-#include "gcs_backend_test.hpp" // must be included last
+#include "gcs_backend_test.hpp"
 
 // Fake backend definitons. Must be global for gcs_backend.c to see
 
@@ -62,37 +66,37 @@ START_TEST (gcs_backend_test)
     long ret;
 
     gu_config_t* config = gu_config_create ();
-    ck_assert(config != NULL);
+    fail_if (config == NULL);
 
     ret = gcs_backend_init (&backend, "wrong://kkk", config);
-    ck_assert(ret == -ESOCKTNOSUPPORT);
+    fail_if (ret != -ESOCKTNOSUPPORT);
 
     ret = gcs_backend_init (&backend, "spread:", config);
-    ck_assert(ret == -EINVAL);
+    fail_if (ret != -EINVAL);
 
     ret = gcs_backend_init (&backend, "dummy://", config);
-    ck_assert_msg(ret == 0, "ret = %ld (%s)", ret, strerror(-ret));
+    fail_if (ret != 0, "ret = %d (%s)", ret, strerror(-ret));
     backend.destroy(&backend);
-//    ck_assert(backend.name == gcs_test_name); this test is broken since we can
+//    fail_if (backend.name != gcs_test_name); this test is broken since we can
 //    no longer use global gcs_dummy_create() symbol because linking with real
 //    gcs_dummy.o
 
     ret = gcs_backend_init (&backend, "gcomm://0.0.0.0:4567", config);
 #ifdef GCS_USE_GCOMM
-    ck_assert_msg(ret == 0, "ret = %d (%s)", ret, strerror(-ret));
-    ck_assert(backend.name == gcs_gcomm_name);
+    fail_if (ret != 0, "ret = %d (%s)", ret, strerror(-ret));
+    fail_if (backend.name != gcs_gcomm_name);
     backend.destroy(&backend);
 #else
-    ck_assert(ret == -ESOCKTNOSUPPORT);
+    fail_if (ret != -ESOCKTNOSUPPORT);
 #endif
 
 //    ret = gcs_backend_init (&backend, "vsbes://kkk");
-//    ck_assert_msg(ret == 0, "ret = %d (%s)", ret, strerror(-ret));
-//    ck_assert(backend.name == gcs_vs_name);
+//    fail_if (ret != 0, "ret = %d (%s)", ret, strerror(-ret));
+//    fail_if (backend.name != gcs_vs_name);
 
 //    ret = gcs_backend_init (&backend, "spread://");
-//    ck_assert_msg(ret == 0, "ret = %d (%s)", ret, strerror(-ret));
-//    ck_assert(backend.name == gcs_spread_name);
+//    fail_if (ret != 0, "ret = %d (%s)", ret, strerror(-ret));
+//    fail_if (backend.name != gcs_spread_name);
 
     gu_config_destroy(config);
 }
